#!/usr/bin/env python
# coding: utf-8

# This script allows the parallel computation of the wind work in the space-frequency domain.
# It allows this computation on a 3600x1800 grid, which is not possible otherwise
# It works with ntasks=16 on nodes=4 with 90GB memory each.
# This skript produces one output per task. Those output files have to be concatenated with processing_concatenate.py Executing this concatenate is best done on a gpu with 150GB


import numpy as np

import xarray as xr
from scipy import signal
import warnings
warnings.filterwarnings('ignore')

from numba import jit, prange
@jit(nopython=False, parallel=True)
def spectral_work(w,p,f,dt):
    B = np.zeros((p.shape[0],p.shape[1],p.shape[2]))
    for jj in range(p.shape[1]):
        if (jj%50 == 0):
            print(jj)
        w_ = w[:,jj,:]
        w_[np.isnan(w_)] = 0.
        w_ = signal.detrend(w_,axis=0,type="linear")
        p_ = p[:,jj,:]
        p_[np.isnan(p_)] = 0.
        p_ = signal.detrend(p_,axis=0,type="linear")
        fw = np.fft.fftshift(np.fft.fft(w_,axis=0),axes=0)
        fg = np.fft.fftshift(np.fft.fft(p_,axis=0),axes=0)

        finput = np.real(fw*fg.conj())*(f[2]-f[1])*dt
        
        B[:,jj,:] = finput
    return B


# In[40]:


idir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"

skip=0 # change to either 0/1 if number of timestep 
p = xr.open_dataset(idir+"p_hydro_atMLB_MAXcap700_202101-202102.nc").p_hydro_atMLB_MAXcap700[skip:,...]
w = xr.open_dataset(idir+"w_p_atMLB_MAXcap700_r01_202101-202102.nc").w_p_atMLB_MAXcap700[skip:,...]

n1 = np.int64(p.shape[0])

print(n1)
dt = 3
freqs = np.fft.fftshift(np.fft.fftfreq(n1,d=dt))


df = freqs[101]-freqs[100]

omega = 1/24 # unit 1/h
cori = (2*omega*np.sin(np.deg2rad(np.array(p.lat))))  # units 1/h
# In[41]:


steps = np.arange(p.shape[2])


# In[42]:


# === mpi4py ===
try:
  from mpi4py import MPI
  comm = MPI.COMM_WORLD
  rank = comm.Get_rank()
  npro = comm.Get_size()
except:
  print('::: Warning: Proceeding without mpi4py! :::')
  rank = 0
  npro = 1
print('proc %d/%d: Hello world!' % (rank, npro))


steps = np.arange(npro)
nstep = p.shape[2]//steps.size

list_all_pros = [0]*npro
for nn in range(npro):
    list_all_pros[nn] = steps[nn::npro]
steps = list_all_pros[rank]

#steps = np.arange(npro)
print(rank,steps)

import sys
outname=sys.argv[1]
outdir="/scratch/m/m300522/"
for nn, step in enumerate(steps):
  # either xarray or netCDF4 to read data for all times at one point in space
    print('proc %d/%d: Step %d/%d' % (rank, npro, step, len(steps)))
    print(step,"step")
    p_cut = p[:,:,step*nstep:(step+1)*nstep]
    w_cut = w[:,:,step*nstep:(step+1)*nstep]
    lon_cut = p.lon[step*nstep:(step+1)*nstep]
#    print(step)
    b = spectral_work(w_cut.data,p_cut.data,freqs,dt)
    print(b.shape)    
    ds = xr.Dataset(
        {
            "new": (["freq","lat", "lon"], b),
        },
        coords={
            "lon": ( lon_cut),
            "lat": ( p.lat),
            "freq": (freqs),
        },
    )
    ds.to_netcdf(outdir+outname+"_"+str(step)+".nc")
