#!/usr/bin/env python
# coding: utf-8

# This script allows the parallel computation of the wind work in the space-frequency domain.
# It allows this computation on a 3600x1800 grid, which is not possible otherwise
# It works with ntasks=16 on nodes=4 with 90GB memory each.
# This skript produces one output per task. Those output files have to be concatenated with processing_concatenateprocessing_concatenate.py Executing this concatenate is best done on a gpu with 150GB


import numpy as np

import xarray as xr
from scipy import signal
import warnings
warnings.filterwarnings('ignore')

from numba import jit, prange
@jit(nopython=False, parallel=True)
def spectral_work(taux,tauy,u,v,f):
    B = np.zeros((taux.shape[0],taux.shape[1],taux.shape[2]))
    for jj in range(taux.shape[1]):
#        if (jj%50 == 0):
 #           print(jj)

        wind = taux[:,jj,:] + tauy[:,jj,:]*1j
        flow = u[:,jj,:] + v[:,jj,:]*1j

        fw = np.fft.fftshift(np.fft.fft(wind,axis=0),axes=0)
        fg = np.fft.fftshift(np.fft.fft(flow,axis=0),axes=0)

        finput = np.real(fw*fg.conj())*(f[2]-f[1])*dt
        
        B[:,jj,:] = finput
    return B



# In[40]:


idir = "/work/mh0256/m300522/dywin/for_paper/feb2021/"

skip=1 # change to either 0/1 if number of timestep 
taux = xr.open_dataset(idir+"stress_x_r01_202101-202102.nc").atmos_fluxes_stress_xw.sel(lon=slice(0,360))[skip:,...]
tauy = xr.open_dataset(idir+"stress_y_r01_202101-202102.nc").atmos_fluxes_stress_yw.sel(lon=slice(0,360))[skip:,...]

u = xr.open_dataset(idir+"u_r01_202101-202102.nc").u.sel(lon=slice(0,360))[skip:,0,...]
v = xr.open_dataset(idir+"v_r01_202101-202102.nc").v.sel(lon=slice(0,360))[skip:,0,...]

n1 = np.int64(u.shape[0])

print(n1)
dt = 1
freqs = np.fft.fftshift(np.fft.fftfreq(n1,d=dt))


df = freqs[101]-freqs[100]

omega = 1/24 # unit 1/h
cori = (2*omega*np.sin(np.deg2rad(np.array(taux.lat))))  # units 1/h
# In[41]:


steps = np.arange(u.shape[2])


# In[42]:


# === mpi4py ===
try:
  from mpi4py import MPI
  comm = MPI.COMM_WORLD
  rank = comm.Get_rank()
  npro = comm.Get_size()
except:
  print('::: Warning: Proceeding without mpi4py! :::')
  rank = 0
  npro = 1
print('proc %d/%d: Hello world!' % (rank, npro))


steps = np.arange(npro)
nstep = taux.shape[2]//steps.size

list_all_pros = [0]*npro
for nn in range(npro):
    list_all_pros[nn] = steps[nn::npro]
steps = list_all_pros[rank]

#steps = np.arange(npro)
print(rank,steps)

import sys
outname=sys.argv[1]
outdir="/scratch/m/m300522/"
for nn, step in enumerate(steps):
  # either xarray or netCDF4 to read data for all times at one point in space
    print('proc %d/%d: Step %d/%d' % (rank, npro, step, len(steps)))
    print(step,"step")
    taux_cut = taux[:,:,step*nstep:(step+1)*nstep]
    tauy_cut = tauy[:,:,step*nstep:(step+1)*nstep]
    v_cut = v[:,:,step*nstep:(step+1)*nstep]
    u_cut = u[:,:,step*nstep:(step+1)*nstep]
    lon_cut = taux.lon[step*nstep:(step+1)*nstep]
#    print(step)
    b = spectral_work(taux_cut.data,tauy_cut.data,u_cut.data,v_cut.data,freqs)
    print(b.shape)    
    ds = xr.Dataset(
        {
            "new": (["freq","lat", "lon"], b),
        },
        coords={
            "lon": ( lon_cut),
            "lat": ( taux.lat),
            "freq": (freqs),
        },
    )
    ds.to_netcdf(outdir+outname+"_"+str(step)+".nc")
